"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ws_1 = __importDefault(require("ws"));
const net_1 = __importDefault(require("net"));
const parser_1 = __importDefault(require("./parser"));
const config_1 = __importDefault(require("./config"));
const tcp = net_1.default.createServer((sock) => {
    console.log("TCP client connected:", sock.remoteAddress);
    function sendBuffer(data) {
        sock.write(new Uint8Array(data));
    }
    function sendString(data) {
        sock.write(new Uint8Array(Buffer.from(data + "\r\n", "utf-8")));
    }
    sendString("[WS-Telnet TCP Proxy]");
    sendString(`Forwarding connection to ${config_1.default.remote}`);
    const ws = new ws_1.default(config_1.default.remote);
    const parser = new parser_1.default();
    ws.on("open", () => {
        console.log("Proxy connection established for client:", sock.remoteAddress);
        parser.on("data", (data) => {
            if (ws.readyState === 1)
                ws.send(data);
        });
        sendString("Proxy connection established.\n");
        sock.on("data", (chunk) => {
            parser.accumulate(chunk);
        });
    });
    ws.on("message", (data) => {
        let buf = undefined;
        if (typeof data === "string") {
            buf = Buffer.from(data, "utf-8");
        }
        else if (data instanceof ArrayBuffer) {
            buf = Buffer.from(data);
        }
        else {
            if (data instanceof Array) {
                data.forEach((d) => sendBuffer(d));
            }
            else {
                buf = data;
            }
        }
        if (buf !== undefined) {
            sendBuffer(buf);
        }
    });
    ws.on("close", (code, reason) => {
        console.log("Proxy connection lost for client:", sock.remoteAddress);
        sock.end();
    });
    sock.on("error", (err) => {
        if (!err.message.includes("ECONNRESET")) {
            throw err;
        }
        else {
            console.log("TCP client connection lost without calling `end`.");
        }
    });
    sock.on("close", (err) => {
        console.log("Client disconnected:", sock.remoteAddress);
        if (ws.readyState === 1) {
            ws.close(undefined, "Local TCP connection lost.");
        }
    });
});
tcp.listen(config_1.default.local.port, config_1.default.local.host);
