"use strict";
const events_1 = require("events");
class TelnetParser extends events_1.EventEmitter {
    constructor() {
        super(...arguments);
        this.buffer = Buffer.alloc(0);
    }
    accumulate(data) {
        if (this.buffer.length === 0)
            this.buffer = data;
        else
            this.buffer = Buffer.concat([this.buffer, data]);
        const buffers = [];
        let iac = this.buffer.indexOf(255);
        if (iac !== -1)
            iac = this.buffer.indexOf(255);
        let behind = 0;
        while (iac !== -1) {
            if (this.buffer[iac + 1] === 255)
                iac += 2;
            else {
                if (this.buffer[iac + 1] === 240)
                    iac += 2;
                buffers.push(this.buffer.slice(behind, iac));
                behind = iac;
                iac = this.buffer.indexOf(255, behind + 1);
            }
        }
        if (behind < this.buffer.length && buffers.length > 0) {
            buffers.push(this.buffer.slice(behind));
        }
        else {
            buffers.push(this.buffer);
        }
        buffers
            .filter((x) => x.length > 0)
            .map((x) => TelnetParser.cleanSubnegotiationData(x))
            .forEach((buff) => {
            if (buff.length > 2 && buff[buff.length - 2] === 0x0d && buff[buff.length - 1] === 0x0a)
                buff = buff.slice(0, buff.length - 2);
            this.emit("data", buff);
        });
        this.buffer = Buffer.alloc(0);
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    emit(event, ...args) {
        return super.emit(event, ...args);
    }
}
(function (TelnetParser) {
    function cleanSubnegotiationData(data) {
        const temp = [];
        for (let i = 0; i < data.byteLength; i++) {
            if (data[i] === 255 && i + 1 < data.byteLength && data[i + 1] === 255)
                i++;
            temp.push(data[i]);
        }
        return Buffer.from(temp);
    }
    TelnetParser.cleanSubnegotiationData = cleanSubnegotiationData;
})(TelnetParser || (TelnetParser = {}));
module.exports = TelnetParser;
